// ignore_for_file: unnecessary_null_comparison

import 'package:flutter/material.dart';
import 'view.dart';

class OwlnextInput extends StatefulWidget {
  /// Owlnext form input
  /// Based on Material design and uses TextFormField
  /// Used to automate all input behavior in one place
  /// 
  /// Can be customized through a variety of properties, like type
  /// [TextInputSubType] for specific behaviors
  const OwlnextInput({
    super.key, 
    this.type = TextInputSubType.text,
    required this.controller,
    this.label,
    this.errorText = '',
    this.minimumLines = 1,
    this.hintText = '',
    this.suffixIcon,
    this.enabled = true,
    this.onChanged,
    this.suffixText,
    this.charsLimit,
    this.isMandatory = false,
    this.infoCallback,
    this.amountMaxLimit,
    this.decimalsMaxLimit,
    this.numberMaxLimit,
    this.numberMinLimit,
    this.focus,
    this.customRegex,
    this.minimumDate,
    this.maximumDate,
    this.onSubmitted,
    this.isInsideList = true,
  }) : 
    assert(amountMaxLimit == null || (amountMaxLimit != null && type == TextInputSubType.amount)),
    assert(decimalsMaxLimit == null || (decimalsMaxLimit != null && type == TextInputSubType.amount)),
    assert(numberMaxLimit == null || (numberMaxLimit != null && type == TextInputSubType.numbers)),
    assert(numberMinLimit == null || (numberMinLimit != null && type == TextInputSubType.numbers)),
    assert(minimumDate == null || (minimumDate != null && type == TextInputSubType.date)),
    assert(maximumDate == null || (maximumDate != null && type == TextInputSubType.date));

  
  /// Text input type, determines behavior depending on type
  final TextInputSubType type;
  /// Text input controller
  final TextEditingController controller;
  /// Is enabled or not
  final bool enabled;
  /// Is mandatory or not, determines the validator behavior
  final bool isMandatory;
  /// Is the text input inside a list, enables the autoscroll on validation
  final bool isInsideList;
  /// Called every time the text input is changed, with the current String as arg
  final void Function(String)? onChanged;
  /// Number of characters limitation, null if none
  final int? charsLimit;
  /// Number of lines for text areas, null if classic input
  final int? minimumLines;
  /// Called when clicking on the information icon, right to the label
  /// Can be anything (openPanel, openPopup...)
  final Function? infoCallback;

  /// Specifies the maximum amount if the input is an amount
  final double? amountMaxLimit;
  /// Specifies the maximum number of decimals, if the input is an amount
  final int? decimalsMaxLimit;
  /// Specifies the maximum number to reach if the input is a number
  final int? numberMaxLimit;
  /// Specifies the minimum number to reach if the input is a number
  final int? numberMinLimit;
  /// Specifies a custom FocusNode for the input
  /// Useful if there is a custom Focus-related behavior
  final FocusNode? focus;
  /// Specifies a custom Regex to check on validation
  final RegExp? customRegex;
  /// Specifies a minimum date to check on validation, if the input is a date
  final DateTime? minimumDate;
  /// Specifies a maximum date to check on validation, if the input is a date
  final DateTime? maximumDate;
  /// Custom callback on input validation
  /// Useful for calling a specific API for example (BAN, SIRET...)
  final void Function(String)? onSubmitted;
  final String? label;
  final String? errorText;
  final String? hintText;
  final Widget? suffixIcon;
  final String? suffixText;

  @override
  OwlnextInputState createState() {
    return OwlnextInputState();
  }
}

class OwlnextInputState extends State<OwlnextInput> with ViewMixin {
  FocusNode focusNode = FocusNode();

  @override
  void dispose() {
    super.dispose();
    focusNode.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

/// All subtypes of text input
/// Determines the input's behavior on change and on form validation
enum TextInputSubType {
  text,
  date,
  shortDate,
  numbers,
  amount,
  email,
  hours,
  password,
}